@@ -24,7 +24,7 @@ from ..mesonlib import (
     is_windows, LibType, OptionKey,
 )
 from .compilers import (Compiler, cuda_buildtype_args, cuda_optimization_args,
-                        cuda_debug_args)
+                        cuda_debug_args, CompileCheckMode)
 
 if T.TYPE_CHECKING:
     from ..build import BuildTarget
@@ -195,7 +195,7 @@ class CudaCompiler(Compiler):
         a double-quoted string a split-point. Single-quotes do not provide protection
         against splitting; In fact, after splitting they are \-escaped. Unfortunately,
         double-quotes don't protect against shell expansion. What follows is a
-        complex dance to accomodate everybody.
+        complex dance to accommodate everybody.
         """
 
         SQ = "'"
@@ -290,8 +290,7 @@ class CudaCompiler(Compiler):
                 raise ValueError("-Xcompiler flag merging failed, unknown argument form!")
         return xflags
 
-    @classmethod
-    def _to_host_flags(cls, flags: T.List[str], phase: _Phase = _Phase.COMPILER) -> T.List[str]:
+    def _to_host_flags(self, flags: T.List[str], phase: _Phase = _Phase.COMPILER) -> T.List[str]:
         """
         Translate generic "GCC-speak" plus particular "NVCC-speak" flags to NVCC flags.
 
@@ -320,7 +319,7 @@ class CudaCompiler(Compiler):
             #     follow the name of the option itself by either one of more spaces or an
             #     equals character. When a one-character short name such as -I, -l, and -L
             #     is used, the value of the option may also immediately follow the option
-            #     itself without being seperated by spaces or an equal character. The
+            #     itself without being separated by spaces or an equal character. The
             #     individual values of list options may be separated by commas in a single
             #     instance of the option, or the option may be repeated, or any
             #     combination of these two cases.
@@ -353,11 +352,10 @@ class CudaCompiler(Compiler):
             # an exception for -D (where this would be value-changing) and -U (because
             # it isn't possible to define a macro with a comma in the name).
 
-            if flag in cls._FLAG_PASSTHRU_NOARGS:
+            if flag in self._FLAG_PASSTHRU_NOARGS:
                 xflags.append(flag)
                 continue
 
-
             # Handle breakup of flag-values into a flag-part and value-part.
             if   flag[:1] not in '-/':
                 # This is not a flag. It's probably a file input. Pass it through.
@@ -384,19 +382,23 @@ class CudaCompiler(Compiler):
                 else:                            # -Isomething
                     val = flag[2:]
                 flag = flag[:2]                  # -I
-            elif flag in cls._FLAG_LONG2SHORT_WITHARGS or \
-                 flag in cls._FLAG_SHORT2LONG_WITHARGS:
+            elif flag in self._FLAG_LONG2SHORT_WITHARGS or \
+                 flag in self._FLAG_SHORT2LONG_WITHARGS:
                 # This is either -o or a multi-letter flag, and it is receiving its
                 # value isolated.
                 try:
                     val = next(flagit)           # -o something
                 except StopIteration:
                     pass
-            elif flag.split('=',1)[0] in cls._FLAG_LONG2SHORT_WITHARGS or \
-                 flag.split('=',1)[0] in cls._FLAG_SHORT2LONG_WITHARGS:
+            elif flag.split('=',1)[0] in self._FLAG_LONG2SHORT_WITHARGS or \
+                 flag.split('=',1)[0] in self._FLAG_SHORT2LONG_WITHARGS:
                 # This is either -o or a multi-letter flag, and it is receiving its
                 # value after an = sign.
                 flag, val = flag.split('=',1)    # -o=something
+            # Some dependencies (e.g., BoostDependency) add unspaced "-isystem/usr/include" arguments
+            elif flag.startswith('-isystem'):
+                val = flag[8:].strip()
+                flag = flag[:8]
             else:
                 # This is a flag, and it's foreign to NVCC.
                 #
@@ -418,16 +420,14 @@ class CudaCompiler(Compiler):
                     xflags.append('-prec-div=true')
                     xflags.append('-Xcompiler='+flag)
                 else:
-                    xflags.append('-Xcompiler='+cls._shield_nvcc_list_arg(flag))
+                    xflags.append('-Xcompiler='+self._shield_nvcc_list_arg(flag))
                     # The above should securely handle GCC's -Wl, -Wa, -Wp, arguments.
                 continue
 
-
             assert val is not None  # Should only trip if there is a missing argument.
 
-
             # Take care of the various NVCC-supported flags that need special handling.
-            flag = cls._FLAG_LONG2SHORT_WITHARGS.get(flag,flag)
+            flag = self._FLAG_LONG2SHORT_WITHARGS.get(flag,flag)
 
             if   flag in {'-include','-isystem','-I','-L','-l'}:
                 # These flags are known to GCC, but list-valued in NVCC. They potentially
@@ -439,10 +439,14 @@ class CudaCompiler(Compiler):
                 # -U with comma arguments is impossible in GCC-speak (and thus unambiguous
                 #in NVCC-speak, albeit unportable).
                 if len(flag) == 2:
-                    xflags.append(flag+cls._shield_nvcc_list_arg(val))
+                    xflags.append(flag+self._shield_nvcc_list_arg(val))
+                elif flag == '-isystem' and val in self.host_compiler.get_default_include_dirs():
+                    # like GnuLikeCompiler, we have to filter out include directories specified
+                    # with -isystem that overlap with the host compiler's search path
+                    pass
                 else:
                     xflags.append(flag)
-                    xflags.append(cls._shield_nvcc_list_arg(val))
+                    xflags.append(self._shield_nvcc_list_arg(val))
             elif flag == '-O':
                 # Handle optimization levels GCC knows about that NVCC does not.
                 if   val == 'fast':
@@ -463,7 +467,7 @@ class CudaCompiler(Compiler):
                 xflags.append(flag)
                 xflags.append(val)
 
-        return cls._merge_flags(xflags)
+        return self._merge_flags(xflags)
 
     def needs_static_linker(self) -> bool:
         return False
@@ -504,7 +508,7 @@ class CudaCompiler(Compiler):
         binname += '_cross' if self.is_cross else ''
         source_name = os.path.join(work_dir, sname)
         binary_name = os.path.join(work_dir, binname + '.exe')
-        with open(source_name, 'w') as ofile:
+        with open(source_name, 'w', encoding='utf-8') as ofile:
             ofile.write(code)
 
         # The Sanity Test for CUDA language will serve as both a sanity test
@@ -577,7 +581,7 @@ class CudaCompiler(Compiler):
 
     def has_header_symbol(self, hname: str, symbol: str, prefix: str,
                           env: 'Environment', *,
-                          extra_args: T.Optional[T.List[str]] = None,
+                          extra_args: T.Union[None, T.List[str], T.Callable[[CompileCheckMode], T.List[str]]] = None,
                           dependencies: T.Optional[T.List['Dependency']] = None) -> T.Tuple[bool, bool]:
         if extra_args is None:
             extra_args = []
@@ -623,12 +627,9 @@ class CudaCompiler(Compiler):
 
         # We must strip the -std option from the host compiler option set, as NVCC has
         # its own -std flag that may not agree with the host compiler's.
-        overrides = {name: opt.value for name, opt in options.items()}
-        overrides.pop(OptionKey('std', machine=self.for_machine,
-                                       lang=self.host_compiler.language), None)
-        host_options = self.host_compiler.get_options().copy()
-        if 'std' in host_options:
-            del host_options['std'] # type: ignore
+        host_options = {key: options.get(key, opt) for key, opt in self.host_compiler.get_options().items()}
+        std_key = OptionKey('std', machine=self.for_machine, lang=self.host_compiler.language)
+        overrides = {std_key: 'none'}
         return OptionOverrideProxy(overrides, host_options)
 
     def get_option_compile_args(self, options: 'KeyedOptionDictType') -> T.List[str]:
@@ -650,11 +651,9 @@ class CudaCompiler(Compiler):
 
     def get_soname_args(self, env: 'Environment', prefix: str, shlib_name: str,
                         suffix: str, soversion: str,
-                        darwin_versions: T.Tuple[str, str],
-                        is_shared_module: bool) -> T.List[str]:
+                        darwin_versions: T.Tuple[str, str]) -> T.List[str]:
         return self._to_host_flags(self.host_compiler.get_soname_args(
-            env, prefix, shlib_name, suffix, soversion, darwin_versions,
-            is_shared_module), _Phase.LINKER)
+            env, prefix, shlib_name, suffix, soversion, darwin_versions), _Phase.LINKER)
 
     def get_compile_only_args(self) -> T.List[str]:
         return ['-c']
@@ -761,3 +760,12 @@ class CudaCompiler(Compiler):
             return [self._shield_nvcc_list_arg('-ccbin='+ccbindir, False)]
         else:
             return []
+
+    def get_profile_generate_args(self) -> T.List[str]:
+        return ['-Xcompiler=' + x for x in self.host_compiler.get_profile_generate_args()]
+
+    def get_profile_use_args(self) -> T.List[str]:
+        return ['-Xcompiler=' + x for x in self.host_compiler.get_profile_use_args()]
+
+    def get_disable_assert_args(self) -> T.List[str]:
+        return self.host_compiler.get_disable_assert_args()
