@@ -13,7 +13,6 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from mesonbuild import coredata
 import os
 import shutil
 import typing as T
@@ -21,11 +20,12 @@ import xml.etree.ElementTree as ET
 
 from . import ModuleReturnValue, ExtensionModule
 from .. import build
+from .. import coredata
 from .. import mlog
 from ..dependencies import find_external_dependency, Dependency, ExternalLibrary
 from ..mesonlib import MesonException, File, FileOrString, version_compare, Popen_safe
-from . import ModuleReturnValue, ExtensionModule
 from ..interpreter import extract_required_kwarg
+from ..interpreter.type_checking import NoneType
 from ..interpreterbase import ContainerTypeInfo, FeatureDeprecated, KwargInfo, noPosargs, FeatureNew, typed_kwargs
 from ..programs import ExternalProgram, NonExistingExternalProgram
 
@@ -44,7 +44,7 @@ if T.TYPE_CHECKING:
         """Keyword arguments for the Resource Compiler method."""
 
         name: T.Optional[str]
-        sources: T.List[FileOrString]
+        sources: T.Sequence[T.Union[FileOrString, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList]]
         extra_args: T.List[str]
         method: str
 
@@ -52,7 +52,7 @@ if T.TYPE_CHECKING:
 
         """Keyword arguments for the Ui Compiler method."""
 
-        sources: T.Sequence[T.Union[FileOrString, build.CustomTarget]]
+        sources: T.Sequence[T.Union[FileOrString, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList]]
         extra_args: T.List[str]
         method: str
 
@@ -60,8 +60,8 @@ if T.TYPE_CHECKING:
 
         """Keyword arguments for the Moc Compiler method."""
 
-        sources: T.List[T.Union[FileOrString, build.CustomTarget]]
-        headers: T.List[T.Union[FileOrString, build.CustomTarget]]
+        sources: T.Sequence[T.Union[FileOrString, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList]]
+        headers: T.Sequence[T.Union[FileOrString, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList]]
         extra_args: T.List[str]
         method: str
         include_directories: T.List[T.Union[str, build.IncludeDirs]]
@@ -93,8 +93,7 @@ if T.TYPE_CHECKING:
         method: str
         qresource: T.Optional[str]
         rcc_extra_arguments: T.List[str]
-        ts_files: T.List[str]
-
+        ts_files: T.List[T.Union[str, File, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList]]
 
 class QtBaseModule(ExtensionModule):
     _tools_detected = False
@@ -103,10 +102,12 @@ class QtBaseModule(ExtensionModule):
     def __init__(self, interpreter: 'Interpreter', qt_version: int = 5):
         ExtensionModule.__init__(self, interpreter)
         self.qt_version = qt_version
-        self.moc: ExternalProgram = NonExistingExternalProgram('moc')
-        self.uic: ExternalProgram = NonExistingExternalProgram('uic')
-        self.rcc: ExternalProgram = NonExistingExternalProgram('rcc')
-        self.lrelease: ExternalProgram = NonExistingExternalProgram('lrelease')
+        self.tools: T.Dict[str, ExternalProgram] = {
+            'moc': NonExistingExternalProgram('moc'),
+            'uic': NonExistingExternalProgram('uic'),
+            'rcc': NonExistingExternalProgram('rcc'),
+            'lrelease': NonExistingExternalProgram('lrelease'),
+        }
         self.methods.update({
             'has_tools': self.has_tools,
             'preprocess': self.preprocess,
@@ -120,13 +121,10 @@ class QtBaseModule(ExtensionModule):
         """Detect Qt (4 or 5) moc, uic, rcc in the specified bindir or in PATH"""
         # It is important that this list does not change order as the order of
         # the returned ExternalPrograms will change as well
-        bins = ['moc', 'uic', 'rcc', 'lrelease']
-        found = {b: NonExistingExternalProgram(name=f'{b}-qt{qt_dep.qtver}')
-                 for b in bins}
         wanted = f'== {qt_dep.version}'
 
         def gen_bins() -> T.Generator[T.Tuple[str, str], None, None]:
-            for b in bins:
+            for b in self.tools:
                 if qt_dep.bindir:
                     yield os.path.join(qt_dep.bindir, b), b
                 # prefer the <tool>-qt<version> of the tool to the plain one, as we
@@ -135,7 +133,7 @@ class QtBaseModule(ExtensionModule):
                 yield b, b
 
         for b, name in gen_bins():
-            if found[name].found():
+            if self.tools[name].found():
                 continue
 
             if name == 'lrelease':
@@ -158,7 +156,7 @@ class QtBaseModule(ExtensionModule):
                                    version_func=get_version,
                                    wanted=wanted)
             if p.found():
-                setattr(self, name, p)
+                self.tools[name] = p
 
     def _detect_tools(self, state: 'ModuleState', method: str, required: bool = True) -> None:
         if self._tools_detected:
@@ -178,10 +176,10 @@ class QtBaseModule(ExtensionModule):
                     mlog.bold('https://bugreports.qt.io/browse/QTBUG-45460'), fatal=False)
         else:
             suffix = f'-qt{self.qt_version}'
-            self.moc = NonExistingExternalProgram(name='moc' + suffix)
-            self.uic = NonExistingExternalProgram(name='uic' + suffix)
-            self.rcc = NonExistingExternalProgram(name='rcc' + suffix)
-            self.lrelease = NonExistingExternalProgram(name='lrelease' + suffix)
+            self.tools['moc'] = NonExistingExternalProgram(name='moc' + suffix)
+            self.tools['uic'] = NonExistingExternalProgram(name='uic' + suffix)
+            self.tools['rcc'] = NonExistingExternalProgram(name='rcc' + suffix)
+            self.tools['lrelease'] = NonExistingExternalProgram(name='lrelease' + suffix)
 
     @staticmethod
     def _qrc_nodes(state: 'ModuleState', rcc_file: 'FileOrString') -> T.Tuple[str, T.List[str]]:
@@ -193,7 +191,7 @@ class QtBaseModule(ExtensionModule):
             abspath = rcc_file.absolute_path(state.environment.source_dir, state.environment.build_dir)
             rcc_dirname = os.path.dirname(abspath)
 
-        # FIXME: what error are we actually tring to check here?
+        # FIXME: what error are we actually trying to check here?
         try:
             tree = ET.parse(abspath)
             root = tree.getroot()
@@ -209,33 +207,41 @@ class QtBaseModule(ExtensionModule):
         except Exception:
             raise MesonException(f'Unable to parse resource file {abspath}')
 
-    def _parse_qrc_deps(self, state: 'ModuleState', rcc_file: 'FileOrString') -> T.List[File]:
-        rcc_dirname, nodes = self._qrc_nodes(state, rcc_file)
+    def _parse_qrc_deps(self, state: 'ModuleState',
+                        rcc_file_: T.Union['FileOrString', build.CustomTarget, build.CustomTargetIndex, build.GeneratedList]) -> T.List[File]:
         result: T.List[File] = []
-        for resource_path in nodes:
-            # We need to guess if the pointed resource is:
-            #   a) in build directory -> implies a generated file
-            #   b) in source directory
-            #   c) somewhere else external dependency file to bundle
-            #
-            # Also from qrc documentation: relative path are always from qrc file
-            # So relative path must always be computed from qrc file !
-            if os.path.isabs(resource_path):
-                # a)
-                if resource_path.startswith(os.path.abspath(state.environment.build_dir)):
-                    resource_relpath = os.path.relpath(resource_path, state.environment.build_dir)
-                    result.append(File(is_built=True, subdir='', fname=resource_relpath))
-                # either b) or c)
+        inputs: T.Sequence['FileOrString'] = []
+        if isinstance(rcc_file_, (str, File)):
+            inputs = [rcc_file_]
+        else:
+            inputs = rcc_file_.get_outputs()
+
+        for rcc_file in inputs:
+            rcc_dirname, nodes = self._qrc_nodes(state, rcc_file)
+            for resource_path in nodes:
+                # We need to guess if the pointed resource is:
+                #   a) in build directory -> implies a generated file
+                #   b) in source directory
+                #   c) somewhere else external dependency file to bundle
+                #
+                # Also from qrc documentation: relative path are always from qrc file
+                # So relative path must always be computed from qrc file !
+                if os.path.isabs(resource_path):
+                    # a)
+                    if resource_path.startswith(os.path.abspath(state.environment.build_dir)):
+                        resource_relpath = os.path.relpath(resource_path, state.environment.build_dir)
+                        result.append(File(is_built=True, subdir='', fname=resource_relpath))
+                    # either b) or c)
+                    else:
+                        result.append(File(is_built=False, subdir=state.subdir, fname=resource_path))
                 else:
-                    result.append(File(is_built=False, subdir=state.subdir, fname=resource_path))
-            else:
-                path_from_rcc = os.path.normpath(os.path.join(rcc_dirname, resource_path))
-                # a)
-                if path_from_rcc.startswith(state.environment.build_dir):
-                    result.append(File(is_built=True, subdir=state.subdir, fname=resource_path))
-                # b)
-                else:
-                    result.append(File(is_built=False, subdir=state.subdir, fname=path_from_rcc))
+                    path_from_rcc = os.path.normpath(os.path.join(rcc_dirname, resource_path))
+                    # a)
+                    if path_from_rcc.startswith(state.environment.build_dir):
+                        result.append(File(is_built=True, subdir=state.subdir, fname=resource_path))
+                    # b)
+                    else:
+                        result.append(File(is_built=False, subdir=state.subdir, fname=path_from_rcc))
         return result
 
     @FeatureNew('qt.has_tools', '0.54.0')
@@ -255,7 +261,7 @@ class QtBaseModule(ExtensionModule):
             mlog.log('qt.has_tools skipped: feature', mlog.bold(feature), 'disabled')
             return False
         self._detect_tools(state, method, required=False)
-        for tool in (self.moc, self.uic, self.rcc, self.lrelease):
+        for tool in self.tools.values():
             if not tool.found():
                 if required:
                     raise MesonException('Qt tools not found')
@@ -267,7 +273,12 @@ class QtBaseModule(ExtensionModule):
     @typed_kwargs(
         'qt.compile_resources',
         KwargInfo('name', str),
-        KwargInfo('sources', ContainerTypeInfo(list, (File, str), allow_empty=False), listify=True, required=True),
+        KwargInfo(
+            'sources',
+            ContainerTypeInfo(list, (File, str, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList), allow_empty=False),
+            listify=True,
+            required=True,
+        ),
         KwargInfo('extra_args', ContainerTypeInfo(list, str), listify=True, default=[]),
         KwargInfo('method', str, default='auto')
     )
@@ -276,13 +287,15 @@ class QtBaseModule(ExtensionModule):
 
         Uses CustomTargets to generate .cpp files from .qrc files.
         """
+        if any(isinstance(s, (build.CustomTarget, build.CustomTargetIndex, build.GeneratedList)) for s in kwargs['sources']):
+            FeatureNew.single_use('qt.compile_resources: custom_target or generator for "sources" keyword argument', '0.60.0', state.subproject)
         out = self._compile_resources_impl(state, kwargs)
         return ModuleReturnValue(out, [out])
 
     def _compile_resources_impl(self, state: 'ModuleState', kwargs: 'ResourceCompilerKwArgs') -> T.List[build.CustomTarget]:
         # Avoid the FeatureNew when dispatching from preprocess
         self._detect_tools(state, kwargs['method'])
-        if not self.rcc.found():
+        if not self.tools['rcc'].found():
             err_msg = ("{0} sources specified and couldn't find {1}, "
                        "please check your qt{2} installation")
             raise MesonException(err_msg.format('RCC', f'rcc-qt{self.qt_version}', self.qt_version))
@@ -294,7 +307,12 @@ class QtBaseModule(ExtensionModule):
         DEPFILE_ARGS: T.List[str] = ['--depfile', '@DEPFILE@'] if self._rcc_supports_depfiles else []
 
         name = kwargs['name']
-        sources = kwargs['sources']
+        sources: T.List['FileOrString'] = []
+        for s in kwargs['sources']:
+            if isinstance(s, (str, File)):
+                sources.append(s)
+            else:
+                sources.extend(s.get_outputs())
         extra_args = kwargs['extra_args']
 
         # If a name was set generate a single .cpp file from all of the qrc
@@ -307,7 +325,7 @@ class QtBaseModule(ExtensionModule):
             rcc_kwargs: T.Dict[str, T.Any] = {  # TODO: if CustomTarget had typing information we could use that here...
                 'input': sources,
                 'output': name + '.cpp',
-                'command': self.rcc.get_command() + ['-name', name, '-o', '@OUTPUT@'] + extra_args + ['@INPUT@'] + DEPFILE_ARGS,
+                'command': self.tools['rcc'].get_command() + ['-name', name, '-o', '@OUTPUT@'] + extra_args + ['@INPUT@'] + DEPFILE_ARGS,
                 'depend_files': qrc_deps,
                 'depfile': f'{name}.d',
             }
@@ -324,7 +342,7 @@ class QtBaseModule(ExtensionModule):
                 rcc_kwargs = {
                     'input': rcc_file,
                     'output': f'{name}.cpp',
-                    'command': self.rcc.get_command() + ['-name', '@BASENAME@', '-o', '@OUTPUT@'] + extra_args + ['@INPUT@'] + DEPFILE_ARGS,
+                    'command': self.tools['rcc'].get_command() + ['-name', '@BASENAME@', '-o', '@OUTPUT@'] + extra_args + ['@INPUT@'] + DEPFILE_ARGS,
                     'depend_files': qrc_deps,
                     'depfile': f'{name}.d',
                 }
@@ -337,26 +355,33 @@ class QtBaseModule(ExtensionModule):
     @noPosargs
     @typed_kwargs(
         'qt.compile_ui',
-        KwargInfo('sources', ContainerTypeInfo(list, (File, str), allow_empty=False), listify=True, required=True),
+        KwargInfo(
+            'sources',
+            ContainerTypeInfo(list, (File, str, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList), allow_empty=False),
+            listify=True,
+            required=True,
+        ),
         KwargInfo('extra_args', ContainerTypeInfo(list, str), listify=True, default=[]),
         KwargInfo('method', str, default='auto')
     )
     def compile_ui(self, state: 'ModuleState', args: T.Tuple, kwargs: 'UICompilerKwArgs') -> ModuleReturnValue:
         """Compile UI resources into cpp headers."""
+        if any(isinstance(s, (build.CustomTarget, build.CustomTargetIndex, build.GeneratedList)) for s in kwargs['sources']):
+            FeatureNew.single_use('qt.compile_ui: custom_target or generator for "sources" keyword argument', '0.60.0', state.subproject)
         out = self._compile_ui_impl(state, kwargs)
         return ModuleReturnValue(out, [out])
 
     def _compile_ui_impl(self, state: 'ModuleState', kwargs: 'UICompilerKwArgs') -> build.GeneratedList:
         # Avoid the FeatureNew when dispatching from preprocess
         self._detect_tools(state, kwargs['method'])
-        if not self.uic.found():
+        if not self.tools['uic'].found():
             err_msg = ("{0} sources specified and couldn't find {1}, "
                        "please check your qt{2} installation")
             raise MesonException(err_msg.format('UIC', f'uic-qt{self.qt_version}', self.qt_version))
 
         # TODO: This generator isn't added to the generator list in the Interpreter
         gen = build.Generator(
-            self.uic,
+            self.tools['uic'],
             kwargs['extra_args'] + ['-o', '@OUTPUT@', '@INPUT@'],
             ['ui_@BASENAME@.h'],
             name=f'Qt{self.qt_version} ui')
@@ -366,27 +391,41 @@ class QtBaseModule(ExtensionModule):
     @noPosargs
     @typed_kwargs(
         'qt.compile_moc',
-        KwargInfo('sources', ContainerTypeInfo(list, (File, str, build.CustomTarget)), listify=True, default=[]),
-        KwargInfo('headers', ContainerTypeInfo(list, (File, str, build.CustomTarget)), listify=True, default=[]),
+        KwargInfo(
+            'sources',
+            ContainerTypeInfo(list, (File, str, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList)),
+            listify=True,
+            default=[],
+        ),
+        KwargInfo(
+            'headers',
+            ContainerTypeInfo(list, (File, str, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList)),
+            listify=True,
+            default=[]
+        ),
         KwargInfo('extra_args', ContainerTypeInfo(list, str), listify=True, default=[]),
         KwargInfo('method', str, default='auto'),
         KwargInfo('include_directories', ContainerTypeInfo(list, (build.IncludeDirs, str)), listify=True, default=[]),
         KwargInfo('dependencies', ContainerTypeInfo(list, (Dependency, ExternalLibrary)), listify=True, default=[]),
     )
     def compile_moc(self, state: 'ModuleState', args: T.Tuple, kwargs: 'MocCompilerKwArgs') -> ModuleReturnValue:
+        if any(isinstance(s, (build.CustomTarget, build.CustomTargetIndex, build.GeneratedList)) for s in kwargs['headers']):
+            FeatureNew.single_use('qt.compile_moc: custom_target or generator for "headers" keyword argument', '0.60.0', state.subproject)
+        if any(isinstance(s, (build.CustomTarget, build.CustomTargetIndex, build.GeneratedList)) for s in kwargs['sources']):
+            FeatureNew.single_use('qt.compile_moc: custom_target or generator for "sources" keyword argument', '0.60.0', state.subproject)
         out = self._compile_moc_impl(state, kwargs)
         return ModuleReturnValue(out, [out])
 
     def _compile_moc_impl(self, state: 'ModuleState', kwargs: 'MocCompilerKwArgs') -> T.List[build.GeneratedList]:
         # Avoid the FeatureNew when dispatching from preprocess
         self._detect_tools(state, kwargs['method'])
-        if not self.moc.found():
+        if not self.tools['moc'].found():
             err_msg = ("{0} sources specified and couldn't find {1}, "
                        "please check your qt{2} installation")
             raise MesonException(err_msg.format('MOC', f'uic-qt{self.qt_version}', self.qt_version))
 
         if not (kwargs['headers'] or kwargs['sources']):
-            raise build.InvalidArguments('At least one of the "headers" or "sources" keyword arguments must be provied and not empty')
+            raise build.InvalidArguments('At least one of the "headers" or "sources" keyword arguments must be provided and not empty')
 
         inc = state.get_include_args(include_dirs=kwargs['include_directories'])
         compile_args: T.List[str] = []
@@ -398,18 +437,18 @@ class QtBaseModule(ExtensionModule):
         arguments = kwargs['extra_args'] + inc + compile_args + ['@INPUT@', '-o', '@OUTPUT@']
         if kwargs['headers']:
             moc_gen = build.Generator(
-                self.moc, arguments, ['moc_@BASENAME@.cpp'],
+                self.tools['moc'], arguments, ['moc_@BASENAME@.cpp'],
                 name=f'Qt{self.qt_version} moc header')
             output.append(moc_gen.process_files(kwargs['headers'], state))
         if kwargs['sources']:
             moc_gen = build.Generator(
-                self.moc, arguments, ['@BASENAME@.moc'],
+                self.tools['moc'], arguments, ['@BASENAME@.moc'],
                 name=f'Qt{self.qt_version} moc source')
             output.append(moc_gen.process_files(kwargs['sources'], state))
 
         return output
 
-    # We can't use typed_pos_args here, the signature is ambiguious
+    # We can't use typed_pos_args here, the signature is ambiguous
     @typed_kwargs(
         'qt.preprocess',
         KwargInfo('sources', ContainerTypeInfo(list, (File, str)), listify=True, default=[], deprecated='0.59.0'),
@@ -469,14 +508,16 @@ class QtBaseModule(ExtensionModule):
         'qt.compile_translations',
         KwargInfo('build_by_default', bool, default=False),
         KwargInfo('install', bool, default=False),
-        KwargInfo('install_dir', str),
+        KwargInfo('install_dir', (str, NoneType)),
         KwargInfo('method', str, default='auto'),
-        KwargInfo('qresource', str, since='0.56.0'),
+        KwargInfo('qresource', (str, NoneType), since='0.56.0'),
         KwargInfo('rcc_extra_arguments', ContainerTypeInfo(list, str), listify=True, default=[], since='0.56.0'),
-        KwargInfo('ts_files', ContainerTypeInfo(list, (str, File)), listify=True, default=[]),
+        KwargInfo('ts_files', ContainerTypeInfo(list, (str, File, build.CustomTarget, build.CustomTargetIndex, build.GeneratedList)), listify=True, default=[]),
     )
     def compile_translations(self, state: 'ModuleState', args: T.Tuple, kwargs: 'CompileTranslationsKwArgs') -> ModuleReturnValue:
         ts_files = kwargs['ts_files']
+        if any(isinstance(s, (build.CustomTarget, build.CustomTargetIndex, build.GeneratedList)) for s in ts_files):
+            FeatureNew.single_use('qt.compile_translations: custom_target or generator for "ts_files" keyword argument', '0.60.0', state.subproject)
         install_dir = kwargs['install_dir']
         qresource = kwargs['qresource']
         if qresource:
@@ -497,26 +538,32 @@ class QtBaseModule(ExtensionModule):
                     ts_files.append(c.rstrip('.qm') + '.ts')
                 else:
                     raise MesonException(f'qt.compile_translations: qresource can only contain qm files, found {c}')
-            results = self.preprocess(state, [], {'qresources': qresource, 'rcc_extra_arguments': kwargs['rcc_extra_arguments']})
+            results = self.preprocess(state, [], {'qresources': qresource_file, 'rcc_extra_arguments': kwargs['rcc_extra_arguments']})
         self._detect_tools(state, kwargs['method'])
         translations: T.List[build.CustomTarget] = []
         for ts in ts_files:
-            if not self.lrelease.found():
+            if not self.tools['lrelease'].found():
                 raise MesonException('qt.compile_translations: ' +
-                                     self.lrelease.name + ' not found')
+                                     self.tools['lrelease'].name + ' not found')
             if qresource:
+                # In this case we know that ts_files is always a List[str], as
+                # it's generated above and no ts_files are passed in. However,
+                # mypy can't figure that out so we use assert to assure it that
+                # what we're doing is safe
+                assert isinstance(ts, str), 'for mypy'
                 outdir = os.path.dirname(os.path.normpath(os.path.join(state.subdir, ts)))
                 ts = os.path.basename(ts)
             else:
                 outdir = state.subdir
-            cmd = [self.lrelease, '@INPUT@', '-qm', '@OUTPUT@']
-            lrelease_kwargs = {'output': '@BASENAME@.qm',
-                               'input': ts,
-                               'install': kwargs.get('install', False),
-                               'build_by_default': kwargs.get('build_by_default', False),
-                               'command': cmd}
-            if install_dir is not None:
-                lrelease_kwargs['install_dir'] = install_dir
+            cmd = [self.tools['lrelease'], '@INPUT@', '-qm', '@OUTPUT@']
+            lrelease_kwargs: T.Dict[str, T.Any] = {
+                'output': '@BASENAME@.qm',
+                'input': ts,
+                'install': kwargs['install'],
+                'install_dir': install_dir or [],
+                'install_tag': 'i18n',
+                'build_by_default': kwargs['build_by_default'],
+                'command': cmd}
             lrelease_target = build.CustomTarget(f'qt{self.qt_version}-compile-{ts}', outdir, state.subproject, lrelease_kwargs)
             translations.append(lrelease_target)
         if qresource:
