@@ -43,7 +43,8 @@ from . import environment
 from . import mlog
 from .coredata import major_versions_differ, MesonVersionMismatchException
 from .coredata import version as coredata_version
-from .mesonlib import MesonException, OrderedSet, get_wine_shortpath, split_args, join_args
+from .mesonlib import (MesonException, OrderedSet, RealPathAction,
+                       get_wine_shortpath, join_args, split_args)
 from .mintro import get_infodir, load_info_file
 from .programs import ExternalProgram
 from .backend.backends import TestProtocol, TestSerialisation
@@ -104,7 +105,7 @@ def add_arguments(parser: argparse.ArgumentParser) -> None:
                         help='List available tests.')
     parser.add_argument('--wrapper', default=None, dest='wrapper', type=split_args,
                         help='wrapper to run tests with (e.g. Valgrind)')
-    parser.add_argument('-C', default='.', dest='wd',
+    parser.add_argument('-C', dest='wd', action=RealPathAction,
                         # https://github.com/python/typeshed/issues/3107
                         # https://github.com/python/mypy/issues/7177
                         type=os.path.abspath,  # type: ignore
@@ -429,7 +430,7 @@ class TAPParser:
             if not line:
                 return
 
-            yield self.Error('unexpected input at line {}'.format((self.lineno,)))
+            yield self.Error(f'unexpected input at line {self.lineno}')
         else:
             # end of file
             if self.state == self._YAML:
@@ -467,7 +468,7 @@ class TestLogger:
 class TestFileLogger(TestLogger):
     def __init__(self, filename: str, errors: str = 'replace') -> None:
         self.filename = filename
-        self.file = open(filename, 'w', encoding='utf8', errors=errors)
+        self.file = open(filename, 'w', encoding='utf-8', errors=errors)
 
     def close(self) -> None:
         if self.file:
@@ -989,7 +990,14 @@ class TestRunGTest(TestRunExitCode):
         if self.test.workdir:
             filename = os.path.join(self.test.workdir, filename)
 
-        self.junit = et.parse(filename)
+        try:
+            self.junit = et.parse(filename)
+        except FileNotFoundError:
+            # This can happen if the test fails to run or complete for some
+            # reason, like the rpath for libgtest isn't properly set. ExitCode
+            # will handle the failure, don't generate a stacktrace.
+            pass
+
         super().complete(returncode, res, stdo, stde)
 
 TestRun.PROTOCOL_TO_CLASS[TestProtocol.GTEST] = TestRunGTest
@@ -1485,8 +1493,7 @@ class TestHarness:
 
         startdir = os.getcwd()
         try:
-            if self.options.wd:
-                os.chdir(self.options.wd)
+            os.chdir(self.options.wd)
             self.build_data = build.load(os.getcwd())
             if not self.options.setup:
                 self.options.setup = self.build_data.test_setup_default_name
@@ -1653,17 +1660,16 @@ class TestHarness:
             # wrapper script.
             sys.exit(125)
 
-        self.name_max_len = max([uniwidth(self.get_pretty_suite(test)) for test in tests])
+        self.name_max_len = max(uniwidth(self.get_pretty_suite(test)) for test in tests)
         startdir = os.getcwd()
         try:
-            if self.options.wd:
-                os.chdir(self.options.wd)
+            os.chdir(self.options.wd)
             runners = []             # type: T.List[SingleTestRunner]
             for i in range(self.options.repeat):
                 runners.extend(self.get_test_runner(test) for test in tests)
                 if i == 0:
-                    self.duration_max_len = max([len(str(int(runner.timeout or 99)))
-                                                 for runner in runners])
+                    self.duration_max_len = max(len(str(int(runner.timeout or 99)))
+                                                 for runner in runners)
                     # Disable the progress report if it gets in the way
                     self.need_console = any(runner.console_mode is not ConsoleUser.LOGGER
                                              for runner in runners)
@@ -1887,7 +1893,10 @@ class TestHarness:
             l.start(self)
 
         if sys.platform != 'win32':
-            asyncio.get_event_loop().add_signal_handler(signal.SIGINT, sigint_handler)
+            if os.getpgid(0) == os.getpid():
+                asyncio.get_event_loop().add_signal_handler(signal.SIGINT, sigint_handler)
+            else:
+                asyncio.get_event_loop().add_signal_handler(signal.SIGINT, sigterm_handler)
             asyncio.get_event_loop().add_signal_handler(signal.SIGTERM, sigterm_handler)
         try:
             for runner in runners:
